﻿/** 
 * @file EventListener.cpp
 * @brief イベントの処理を記述する
 * @author Iwata Naoki
 * @date 2011-03-19
 * @version 1.00
 * @version 1.01 2013/03/21 katayama
 * @version 1.02 2014/09/30 katayama
 * @version 1.03 2014/10/02 katayama
 * @version 1.10 2014/10/06 katayama
 * @version 1.11 2015/02/09 katayama
 * @version 1.12 2015/05/21 katayama
 *
 * Copyright (C) 2010 - TPIP User Community All rights reserved.
 * このファイルの著作権は、TPIPユーザーコミュニティの規約に従い使用許諾をします。
 */


/*-------- include --------*/
#include "TPIP3.h"           // TPIP関係
#include "EventListener.h"   // ウィンドウのイベント処理関係

#include <stdio.h>
#pragma  warning(disable:4996)	//	セキュリティが弱い古い関数に対する警告の解除
#include "def_Share.h"
//#include "ShareMemory.h"
#include "sub_module.h"
#include "rShare.h"
#include "TPIP3APP_load.h"

#include "rLOG_cl.h"
#define rLOG_PORT 9801		// rLOG TCP port no

/*-------- メンバ変数定義 --------*/
static struct GC_STRUCT  mPad;		//!< @brief ゲームパッド入力データ
static tpip2pc_dt_t sens_dt;		// sensor data (rShare)
static pc2tpip_dt_t ctrl_dt;		// control data(rShare)
static tpip_share_t share_dt;		// windows shared memory(thisAPP -> otherAPP)
static int PI_rq_reset;				// PI & IMU reset request flag

static int sens_size = 0;
static int ctrl_size = 0;
static int ShareM_res = 0;

static int    updatePaint;			//!< @brief onPaint 更新フラグ(0:更新済　1:未更新)

static int tpip_stat = 0;
static int tpip_cnt  = 0;
static char TPLX_flname[] = "TPLX_prog";	// TPLX APP file name

/*-------- 関数定義 --------*/

/**
 * WM_CREATEのメッセージクラッカー\n
 * ウィンドウが生成される前にコールされる関数\n
 * ここで生成したタイマーを利用し、TPIPとの通信、画面の描画のイベントを発生します
 *
 * @param[in]  hwnd            イベントが発生したウィンドウのハンドル
 * @param[in]  lpCreateStruct  ウィンドウ作成時のパラメータ
 *
 * @retval 常にTRUE
 *
 * @see http://msdn.microsoft.com/en-us/library/ms632619(VS.85).aspx
 *
 */
BOOL onCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct) {

	TPIP3APP_load(IP, TPLX_flname);				// TPLX APP load & run

	ShareM_res = (int)TPLX_CreateShareFile();	// create windows Shared memory

	rSHA_init(IP, RSHARE_PORT);					// Open remote Share memory

//	rLOG_init(IP, rLOG_PORT, "log_file.txt");	// remote logging initialize
	rLOG_init(IP, rLOG_PORT, "log_file%04d%02d%02d_%02d%02d%02d.txt");	// remote logging initialize

	SetTimer(hwnd, ID_MYTIMER, 50, NULL);	// タイマーを生成する
	return TRUE; //ここをFALSE(0)にするとメッセージでDestroyが呼ばれる
}


/**
 * WM_ACTIVATEのメッセージクラッカー\n
 * ウィンドウがアクティブ/非アクティブになる際にコールされる関数
 *
 * @param[in] hwnd          イベントが発生したウィンドウのハンドル
 * @param[in] state         ウィンドウがアクティブ/非アクティブかの状態
 * @param[in] hwndActDeact  ウィンドウのハンドル\n
 *                            state == 0の場合は、新しくアクティブになったウィンドウのハンドル\n
 *                            state != 0の場合は、それまでアクティブであったウィンドウのハンドル
 * @param[in] fMinimized    ウィンドウが最小化されているかの判定フラグ\n
 *                            0以外の値の場合はウィンドウは最小化されています\n
 *                            0の場合はウィンドウは最小化されていません\n
 *
 * @see http://msdn.microsoft.com/ja-jp/library/ms646274(d=lightweight).aspx
 *
 */
void onActivate(HWND hwnd, UINT state, HWND hwndActDeact, BOOL fMinimized) {
	return;
}



/**
 * WM_CLOSEのメッセージクラッカー\n
 * ウィンドウかアプリケーションが終了する際にコールされる関数\n
 * 終了の確認を行い、YESならばウィンドウを破棄する。NOならば破棄しない
 *
 * @param[in] hwnd  イベントが発生したウィンドウのハンドル
 *
 * @see http://msdn.microsoft.com/en-us/library/ms632617(VS.85).aspx
 *
 */
void onClose(HWND hwnd) {
	
	DestroyWindow(hwnd); // ウインドウを閉じる関数

	return;
}



/**
 * WM_DESTROYのメッセージクラッカー\n
 * ウィンドウが破棄される際にコールされる関数\n
 * TPIPライブラリと生成したタイマーの解放を行う
 *
 * @param[in]  hwnd イベントが発生したウィンドウのハンドル
 *
 * @see http://msdn.microsoft.com/en-us/library/ms632620(VS.85).aspx
 *
 */
void onDestroy(HWND hwnd) {

	rSHA_close();			// Close remote Shared memory

	if (ShareM_res) {
		TPLX_CloseShareFile();	// close windows Shared memory
		ShareM_res = 0;
	}

	rLOG_stop();			// remote Logging stop
	Sleep(100);				// wait 100ms
	rLOG_end();				// remote Logging close
	TPIP3APP_end(TPLX_flname);	// TPLX APP kill

	// TPIPライブラリ群の解放
	TPGM_delete();
	TPGC_close();
	TPJT_close();
	Font_del();

	//タイマー解放
	KillTimer(hwnd, ID_MYTIMER);
	PostQuitMessage(0);

	return;
}



/**
 * WM_SIZEのメッセージクラッカー\n
 * ウィンドウのサイズが変更された後にコールされる関数\n
 * TPGMで使用しているサーフェイスを変更後のサイズで構成し直す
 *
 * @param[in]  hwnd  イベントが発生したウィンドウのハンドル
 * @param[in]  state サイズ変更のタイプ
 * @param[in]  cx    ウィンドウの横幅
 * @param[in]  cy    ウィンドウの縦幅
 *
 * @see http://msdn.microsoft.com/en-us/library/ms632646(v=vs.85).aspx
 *
 */
void onSize(HWND hwnd, UINT state, int cx, int cy) {
		
	// サーフェイスの再構成
	TPGM_delete();
	TPGM_create(hwnd, QVGA, NULL);

	return;
}



/**
 * WM_TIMERのメッセージクラッカー\n
 * タイマーが終了した際にコールされる関数\n
 * ゲームパッドからの入力、TPIPとの通信、画面の描画のイベントを発生する\n
 * 制御データの加工等はTPJT_set_ctrl()を実行するまでに行うこと
 *
 * @param[in]  hwnd イベントが発生したウィンドウのハンドル
 * @param[in]  id   イベントを起こしたタイマーID
 *
 * @see http://msdn.microsoft.com/en-us/library/ms644902(VS.85).aspx
 *
 */
void onTimer(HWND hwnd, UINT id) {

	if (id == ID_MYTIMER) {		// 予期しないタイマーからのイベントは無視する

		tpip_stat = TPJT_get_com_stat();	// TPIPの通信ステータスを取得
		tpip_cnt++;
		
		TPGC_get(&mPad);		// ゲームパッドからの入力を取得

		// remote Shared memory
		sens_size = rSHA_recv((char*)&sens_dt, sizeof(sens_dt));

		for (int i = 0; i < 8; i++) {
			ctrl_dt.axis[i] = mPad.axis[i];	// ゲームコントローラの情報をTPIP3指令情報にセット
		}
		ctrl_dt.BTN = mPad.BTN_bit;
		ctrl_size = rSHA_send((char*)&ctrl_dt, sizeof(ctrl_dt));

		if (ShareM_res) {
			if (sens_size) {
				if (TPLX_IsRenewData() == 0) {
					share_dt.s.imu = sens_dt.imu;
					for (int i = 0; i < 16; i++) share_dt.s.tpip_mon.AI[i] = sens_dt.AI[i];
					share_dt.s.status._bits.pc  = 1;
					share_dt.s.is_new = 1;
					TPLX_SetShareData(share_dt.buff);
				}
			}
		}

		ctrl_dt.PI_reset._dword = 0;

		if (PI_rq_reset) {
			ctrl_dt.PI_reset._dword = PI_rq_reset;
			PI_rq_reset = 0;
		}

		/*
			画面描画更新の監視処理
			タイマーイベント期間内に画面更新が無かった場合WM_PAINTメッセージを送信を行う。
		*/
		if (hwnd != NULL) {		// ウィンドウが生成されているか？
			if (!updatePaint) {	// onPaint 更新フラグが未更新か？
				InvalidateRect(hwnd, NULL, FALSE);	// WM_PAINTメッセージを送信
			}
			updatePaint = 0;	//  onPaint 更新フラグをRESET
		} 
	}

	return;
}



/** 
 * WM_KEYDOWNのメッセージクラッカー\n
 * システムキー以外のキーが押された際にコールされる関数\n
 * システムキーとはAltキーが押されていない状態のキーの事を言う
 *
 * @param[in]  hwnd    イベントが発生したウィンドウのハンドル
 * @param[in]  vk      仮想キーコード
 * @param[in]  fDown   常にTRUE
 * @param[in]  cRepeat メッセージのリピート回数
 * @param[in]  flags   lParamのHIWORDに定義されているフラグ
 *
 * @see http://msdn.microsoft.com/en-us/library/ms646280(VS.85).aspx
 *
 */
void onKeydown(HWND hwnd, UINT vk, BOOL fDown, int cRepeat, UINT flags) {

	ctrl_dt.key_code = vk;	// key code -> rShare

	switch(vk) {
		case 'R':		// 'R'キー
			if ((rLOG_get_stat() & 0x0f0)) rLOG_stop();	// remote logging stop
			else rLOG_start();							// remote logging start
			return;
		case 'Z':		// 'Z'キー
			PI_rq_reset = 0x1FF;
			return;
		case VK_ESCAPE: // ESCキー
		case VK_F12:    // F12
			onClose(hwnd);	// ウインドウを閉じる処理
			return;

		default: // 通常
			return;
	}

	return;
}



/** 
 * WM_KEYUPのメッセージクラッカー\n
 * システムキー以外のキーが離された際にコールされる関数\n
 * システムキーとはAltキーが押されていない状態のキーの事を言う
 *
 * @param[in]  hwnd    イベントが発生したウィンドウのハンドル
 * @param[in]  vk      仮想キーコード
 * @param[in]  fDown   常にFALSE
 * @param[in]  cRepeat メッセージのリピート回数
 * @param[in]  flags   lParamのHIWORDに定義されているフラグ
 *
 * @see http://msdn.microsoft.com/en-us/library/ms646281(VS.85).aspx
 *
 */
void onKeyup(HWND hwnd, UINT vk, BOOL fDown, int cRepeat, UINT flags) {
	return;
}


/** 
 * WM_PAINTのメッセージクラッカー\n
 * システム、または他のアプリケーションから画面の更新要求があった際にコールされる関数\n
 * Jpegデータの取出し、画面描画を行う
 *
 * @param[in]  hwnd  イベントが発生したウィンドウのハンドル
 *
 * @see http://msdn.microsoft.com/en-us/library/dd145213(VS.85).aspx
 *
 */
void onPaint(HWND hwnd) {
	
	HDC   hdc ;       // デバイスコンテキスト
	//RECT  dst, src = {0, 0, 640, 480};
	void* mJpegData;  // JPEGデータ格納変数
	int   mJpegSize;  // JPEGデータサイズ
	int   i,x,y;
	HFONT  hFontOld;

	updatePaint = 1;			// onPaint 更新フラグを更新済に

	mJpegData = TPJT_get_jpeg_file(0, 0, &mJpegSize);	// JPEGデータの取出し
	
	if ((mJpegData)&&(mJpegSize > 0)) {	// JPEGデータが有りの場合
		TPGM_decode(mJpegData, mJpegSize);	// JPEGデータをカメラ画像にデコードする
	}
	TPJT_free_jpeg_file();					// JPEGデータの解放


	TPGM_copy(0, 0);			// ウィンドウ座標(0,0)位置に固定サイズ(480x640)でカメラ映像を描画する 
	//TPGM_copy2(&src, &dst);	// サイズを変更してカメラ映像を描画する時に使用する。
	
	hdc = TPGM_getDC();			// デバイスコンテキストを取得する

	char msg[256];								// 文字列格納変数定義
	SetBkColor(hdc, RGB(0, 0, 0));				// 文字背景色指定
	SetTextColor(hdc, RGB( 255, 255, 255));		// 文字色指定（白）
	hFontOld = (HFONT)SelectObject( hdc, Font_set(2));	// 表示フォント指定

//	sprintf(msg, "PWM 0 = %6d  ", mOutDt.PWM[0]);
	x = 10; y = 480;
	for ( i = 0; i < 8; i++) {					// position 表示
		sprintf(msg, "pos %d = %6d  ", i, ctrl_dt.pos[i]);
		TextOut(hdc, x, y, msg, lstrlen(msg));
		y += 20;
	}
	sprintf(msg, "Reset = %08X  ", ctrl_dt.PI_reset._dword);
	TextOut(hdc, x, y, msg, lstrlen(msg));

	x += 130; y = 480;
	for ( i = 0; i < 8; i++) {					// AI 表示
		sprintf(msg, "AI %d = %6d  ", i, sens_dt.AI[i]);
		TextOut(hdc, x, y, msg, lstrlen(msg));	// 文字表示
		y += 20;
	}

	x += 130; y = 480;
	for ( i = 8; i < 16; i++) {					// AI 表示
		sprintf(msg, "AI %2d = %6d  ", i, sens_dt.AI[i]);
		TextOut(hdc, x, y, msg, lstrlen(msg));	// 文字表示
		y += 20;
	}

	x += 130; y = 480;
	sprintf(msg, "DI   =   %04X  ", sens_dt.DI);	// DI 表示
	TextOut(hdc, x, y, msg, lstrlen(msg));		// 文字表示
	y += 20;

	sprintf(msg, "DO   =   %04X  ", sens_dt.d_out);	// DO 表示
	TextOut(hdc, x, y, msg, lstrlen(msg));		// 文字表示
	y += 20;
	y += 20;
	sprintf(msg, "BATT = %2.2fV  ", (float)(sens_dt.batt)/100);	// Battery 表示
	TextOut(hdc, x, y, msg, lstrlen(msg));		// 文字表示
	y += 20;

	x += 130; y = 480;
	for ( i = 0; i < 8; i++) {					// PI 表示
		sprintf(msg, "PI %d = %6d  ", i, sens_dt.PI[i]);
		TextOut(hdc, x, y, msg, lstrlen(msg));	// 文字表示
		y += 20;
	}

	x += 130; y = 480;
	for ( i = 0; i < 8; i++) {					// PWM montor表示
		sprintf(msg, "PWM %d = %6d  ", i, sens_dt.PWM[i]);
		TextOut(hdc, x, y, msg, lstrlen(msg));	// 文字表示
		y += 20;
	}

	x = 700; y = 10;
	for ( i = 0; i < 8; i++) {					// game controller axis表示
		sprintf(msg, "axis%d = %6d  ", i, ctrl_dt.axis[i]);
		TextOut(hdc, x, y, msg, lstrlen(msg));	// 文字表示
		y += 20;
	}
	sprintf(msg, "button = %04X  ", ctrl_dt.BTN);	// button 表示
	TextOut(hdc, x, y, msg, lstrlen(msg));		// 文字表示

	x = 700; y += 40;
	strcpy(msg, "IMU data");				// IMU data表示
	TextOut(hdc, x, y, msg, lstrlen(msg));	// タイトル

	x = 700; y += 20;
	sprintf(msg, "gyro = ");
	TextOut(hdc, x, y, msg, lstrlen(msg));
	x = 750;
	for ( i = 0; i < 3; i++) {
		sprintf(msg, "%7.3f  ", sens_dt.imu.gyro[i]);
		TextOut(hdc, x+i*60, y, msg, lstrlen(msg));	// gyro
	}

	x = 700; y += 20;
	sprintf(msg, "acc  = ");
	TextOut(hdc, x, y, msg, lstrlen(msg));
	x = 750;
	for ( i = 0; i < 3; i++) {
		sprintf(msg, "%7.3f  ", sens_dt.imu.acc[i]);
		TextOut(hdc, x+i*60, y, msg, lstrlen(msg));	// acc
	}

	x = 700; y += 20;
	sprintf(msg, "mag  = ");
	TextOut(hdc, x, y, msg, lstrlen(msg));
	x = 750;
	for ( i = 0; i < 3; i++) {
		sprintf(msg, "%7.3f  ", sens_dt.imu.mag[i]);
		TextOut(hdc, x+i*60, y, msg, lstrlen(msg));	// mag
	}

	x = 700; y += 20;
	sprintf(msg, "quat = ");
	TextOut(hdc, x, y, msg, lstrlen(msg));
	x = 750;
	for ( i = 0; i < 4; i++) {
		sprintf(msg, "%7.3f  ", sens_dt.imu.quat[i]);
		TextOut(hdc, x+i*60, y, msg, lstrlen(msg));	// quat
	}

	// REC start/stop 表示
	char* REC_msg = "disconnect ";
	int stat = (rLOG_get_stat() & 0x00f);
	if (stat == 1) REC_msg = "REC stop   ";
	if (stat == 2) REC_msg = "REC start  ";
	x = 660; y = 440;
	sprintf(msg, "[ %s ](%02X)  push R key ", REC_msg, stat);
	TextOut(hdc, x, y, msg, lstrlen(msg));		// 文字表示

	x = 660; y = 420;
	sprintf(msg, "TPIP3 status : %02X[%08X] ", tpip_stat, tpip_cnt);
	TextOut(hdc, x, y, msg, lstrlen(msg));		// 文字表示


	SelectObject( hdc, hFontOld);			// 表示フォントを元に戻す
	TPGM_releaseDC();			// デバイスコンテキストを解放
	TPGM_screen();				// スクリーンへ描画
	
	ValidateRect(hwnd, NULL);	// 画面更新を適用する

	return;
}



/** 
 * WM_LBUTTONDOWNのメッセージクラッカー\n
 * ウィンドウ内でマウスの左ボタンを押し込んだ際にコールされる関数
 * 
 * @param[in]  hwnd          イベントが発生したウィンドウのハンドル
 * @param[in]  fDoubleClick  常にFALSE
 * @param[in]  x             イベントが発生した際のマウスのX座標
 * @param[in]  y             イベントが発生した際のマウスのY座標
 * @param[in]  keyFlags      押されている仮想キーのフラグ
 *
 * @see http://msdn.microsoft.com/en-us/library/ms645607(VS.85).aspx
 *
 */
void onLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags) {
	return;
}



/** 
 * WM_LBUTTONUPのメッセージクラッカー\n
 * ウィンドウ内でマウスの左ボタンを離した際にコールされる関数
 * 
 * @param[in]  hwnd      イベントが発生したウィンドウのハンドル
 * @param[in]  x         イベントが発生した際のマウスのX座標
 * @param[in]  y         イベントが発生した際のマウスのY座標
 * @param[in]  keyFlags  押されている仮想キーのフラグ
 *
 * @see http://msdn.microsoft.com/en-us/library/ms645608(VS.85).aspx
 *
 */
void onLButtonUp(HWND hwnd, int x, int y, UINT keyFlags) {
	return;
}



/** 
 * WM_LBUTTONDBLCLKのメッセージクラッカー\n
 * ウィンドウ内でマウスの左ボタンをダブルクリックした際にコールされる関数
 *
 * @param[in]  hwnd          イベントが発生したウィンドウのハンドル
 * @param[in]  fDoubleClick  常にTRUE
 * @param[in]  x             イベントが発生した際のマウスのX座標
 * @param[in]  y             イベントが発生した際のマウスのY座標
 * @param[in]  keyFlags      押されている仮想キーのフラグ
 *
 * @see http://msdn.microsoft.com/en-us/library/ms645606(VS.85).aspx
 *
 */
void onLButtonDblClk(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags) {
	return;
}



/** 
 * WM_LBUTTONDOWNのメッセージクラッカー\n
 * ウィンドウ内でマウスの右ボタンを押し込んだ際にコールされる関数
 * 
 * @param[in]  hwnd          イベントが発生したウィンドウのハンドル
 * @param[in]  fDoubleClick  常にFALSE
 * @param[in]  x             イベントが発生した際のマウスのX座標
 * @param[in]  y             イベントが発生した際のマウスのY座標
 * @param[in]  keyFlags      押されている仮想キーのフラグ
 *
 * @see http://msdn.microsoft.com/en-us/library/ms646242(VS.85).aspx
 *
 */
void onRButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags) {
	return;
}



/** 
 * WM_RBUTTONUPのメッセージクラッカー\n
 * ウィンドウ内でマウスの右ボタンを離した際にコールされる関数
 * 
 * @param[in]  hwnd      イベントが発生したウィンドウのハンドル
 * @param[in]  x         イベントが発生した際のマウスのX座標
 * @param[in]  y         イベントが発生した際のマウスのY座標
 * @param[in]  keyFlags  押されている仮想キーのフラグ
 *
 * @see http://msdn.microsoft.com/en-us/library/ms646243(VS.85).aspx
 *
 */
void onRButtonUp(HWND hwnd, int x, int y, UINT keyFlags) {
	return;
}



/** 
 * WM_RBUTTONDOWNのメッセージクラッカー\n
 * ウィンドウ内でマウスの右ボタンをダブルクリックした際にコールされる関数
 *
 * @param[in]  hwnd          イベントが発生したウィンドウのハンドル
 * @param[in]  fDoubleClick  常にTRUE
 * @param[in]  x             イベントが発生した際のマウスのX座標
 * @param[in]  y             イベントが発生した際のマウスのY座標
 * @param[in]  keyFlags      押されている仮想キーのフラグ
 *
 * @see http://msdn.microsoft.com/en-us/library/ms646241(VS.85).aspx
 *
 */
void onRButtonDblClk(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags) {
	return;
}



/** 
 * WM_MOUSEMOVEのメッセージクラッカー\n
 * ウィンドウ内でマウスが動いた際にコールされる関数
 *
 * @param[in]  hwnd      イベントが発生したウィンドウのハンドル
 * @param[in]  x         イベントが発生した際のマウスのX座標
 * @param[in]  y         イベントが発生した際のマウスのY座標
 * @param[in]  keyFlags  押されている仮想キーのフラグ
 *
 * @see http://msdn.microsoft.com/en-us/library/ms645616(VS.85).aspx
 *
 */
void onMouseMove(HWND hwnd, int x, int y, UINT keyFlags) {
	return;
}

